/**
 * \file 
 * \author lcs09
 *
 * \brief Test suite per le funzioni in ptree.h
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "CUnit.h"
#include "TestDB.h"
#include "Basic.h"
#include "ptree.h"


/** nome file test lettura */
#define FILE_TEST1 "./perm1.dat"
/** nome file test scrittura*/
#define FILE_TEST2 "./perm2.dat"

/** targhe con associato un orario per cui esiste l'autorizzazione e uno
    per cui non esiste */
char* orariOkBad[] = {
  "CN445JK 12/12/2008-13:40 13/12/2008-18:30",
  "DD115PU 01/01/2009-01:00 30/06/2009-03:00",
  "DD115PU 01/01/2008-04:00 30/08/2008-23:50",
  "AS105SU 13/01/2008-01:00 30/09/2008-23:15",
  "BU995PK 01/03/2009-10:10 01/03/2009-22:30",
  "KK876VB 01/02/2009-20:00 28/02/2009-23:00",
   NULL
};

/** targhe non esistenti */
char* orariBadBad[] = {
  "CC445JK 12/12/2008-13:40 13/12/2008-18:30",
  "ZZ005JK 12/12/2008-13:40 13/12/2008-18:30",
   NULL
};

/** esempi di permessi ben formati */
char* permessiok[] = {
  "CN445JK 12/12/2008-13:30 13/12/2008-14:30",
  "DD115PU 01/01/2009-00:00 30/06/2009-00:00",
  "CN445JK 10/02/2009-13:30 11/02/2009-14:30",
  "DD115PU 01/01/2008-00:00 30/08/2008-23:10",
  "AS105SU 13/01/2008-00:00 30/09/2008-20:15",
  "BU995PK 01/03/2009-10:00 01/03/2009-20:30",
  "KK876VB 01/02/2009-00:00 28/02/2009-22:00",
  "KK876VB 01/04/2009-00:00 30/04/2009-22:00",
  "DS009LU 01/01/2009-00:00 30/08/2009-23:10",
  "KC044ZZ 21/01/2009-09:00 22/01/2009-22:00",
  "AU663XZ 21/01/2009-09:00 22/01/2009-22:00",
  NULL
};

/** Funzione di inizializzazione della suite */
int success_init(void) { return 0; }
/** Funzione di clean up della suite */
int success_clean(void) { return 0; }

/**
 * Questa funzione crea un albero di permessi, e poi lo distrugge, 
 * testando le funzioni addPerm, removePerm, checkPerm
 *
 */
void run_testAddCheckRemove (void) {
  int i, a, b;
  nodo_t* ptree = NULL, *p;
  permesso_t* pr=NULL;

  for(i=0;permessiok[i]!=NULL;i++) {
    /* 
       converto il permesso e lo inserisco nell'albero
    */
    pr = convertiRecord(permessiok[i]);
    a = addPerm(&ptree,pr);
    CU_ASSERT_EQUAL_FATAL(a,0);

    /* 
       c'e'?
    */
    p = findPerm(pr,ptree);
    CU_ASSERT_NOT_EQUAL_FATAL(p,NULL);

    b = strcmp(p->targa,pr->targa);
    CU_ASSERT_EQUAL_FATAL(b,0);

    free(pr);
    pr = NULL;

  }

  /* 
     test su stampa albero 
  */
  printTree(NULL);
  printTree(ptree);

  /* 
     test funzione di check con orari dentro/fuori 
     l'intervallo permesso 
  */
  for(i=0;orariOkBad[i]!=NULL;i++) {
    
    pr = convertiRecord(orariOkBad[i]);
    a = checkPerm(pr->targa, pr->in.inizio,ptree);
    CU_ASSERT_EQUAL_FATAL(a,1);

    a = checkPerm(pr->targa, pr->in.fine,ptree);
    CU_ASSERT_EQUAL_FATAL(a,0);

    free(pr);
    pr = NULL;
    }
  
  /*
    ricerca targhe non presenti
  */
  for(i=0;orariBadBad[i]!=NULL;i++) {
    pr = convertiRecord(orariBadBad[i]);
    a = checkPerm(pr->targa, pr->in.fine,ptree);
    CU_ASSERT_EQUAL_FATAL(a,0);

    free(pr);
    pr = NULL;
 
  }

  /* 
     rimozione dei permessi dall'albero 
  */
  for(i=0;permessiok[i]!=NULL;i++) {
    pr = convertiRecord(permessiok[i]);
    a =  removePerm(pr,&ptree);
    CU_ASSERT_EQUAL_FATAL(a,0);

    /* 
       c'e' ancora?
    */
    p = findPerm(pr,ptree);
    CU_ASSERT_EQUAL_FATAL(p,NULL);
    
    
    free(pr);
    pr = NULL;
    
  }
}

/**
 * Questa funzione legge e converte un file di permessi verificando 
 * il funzionamento delle varie funzioni
 *
 * \param fin file di ingresso
 * \param fout di uscita (voci converitite)
 */
void run_testleggiscrivi (char *fin, char* fout) {
    FILE *fsa;
    nodo_t* ptree=NULL;
    int i;
    int a = 0; /* conta i permessi letti  da file */
    int b = 0; /* conta i permessi scritti su file */

    /*
     * Apertura del file dei permessi
     */
    fsa = fopen(fin, "r");
    CU_ASSERT_PTR_NOT_NULL_FATAL(fsa); 


    /*
     * Lettura e caricamento permessi nell'albero
     */
    a = loadPerm(fsa,&ptree);
    CU_ASSERT_NOT_EQUAL_FATAL(a,0);
    CU_ASSERT_NOT_EQUAL_FATAL(a,-1);

    fclose(fsa);
    printf("Letti %d permessi\n",a);

    /* 
       Test la funzione di check dei permessi 
       con orari dentro/fuori 
       l'intervallo permesso
    */

    for(i=0;orariOkBad[i]!=NULL;i++) {
      permesso_t* pr;
      int e;

      pr = convertiRecord(orariOkBad[i]);
      CU_ASSERT_PTR_NOT_NULL_FATAL(pr); 
      
      e = checkPerm(pr->targa, pr->in.inizio,ptree);
      CU_ASSERT_EQUAL_FATAL(e,1);

      e = checkPerm(pr->targa, pr->in.fine,ptree);
      CU_ASSERT_EQUAL_FATAL(e,0);

      free(pr);
    }
    
    /* 
       Test la funzione di ricerca dei permessi
    */

    for(i=0;permessiok[i]!=NULL;i++) {
      permesso_t* pr;
      nodo_t* t;

      pr = convertiRecord(permessiok[i]);
      CU_ASSERT_PTR_NOT_NULL_FATAL(pr); 

      t = findPerm(pr,ptree);
      CU_ASSERT_NOT_EQUAL_FATAL(t,NULL);

      free(pr);
    }

    /*
      Test scrittura nel file dei permessi
    */

    printf("Registro i permessi.\n");
    fsa=fopen(fout,"w");
    CU_ASSERT_PTR_NOT_NULL_FATAL(fsa); 
    b = storePerm(fsa,ptree);
    CU_ASSERT_EQUAL_FATAL(a,b);
    fclose(fsa);
    printf("Registrati %d permessi.\n",b);

    freeTree(ptree);
    ptree = NULL;

    /*
      Test rilettura file
    */
    fsa = fopen(fout, "r");
    CU_ASSERT_PTR_NOT_NULL_FATAL(fsa); 


    /*
     * Lettura e caricamento permessi nell'albero
     */
    a = loadPerm(fsa,&ptree);
    CU_ASSERT_NOT_EQUAL_FATAL(a,0);
    CU_ASSERT_NOT_EQUAL_FATAL(a,-1);

    fclose(fsa);
    printf("Letti %d permessi\n",a);


    for(i=0;permessiok[i]!=NULL;i++) {
      permesso_t* pr;
      nodo_t* t;

      pr = convertiRecord(permessiok[i]);
      CU_ASSERT_PTR_NOT_NULL_FATAL(pr); 

      t = findPerm(pr,ptree);
      CU_ASSERT_NOT_EQUAL_FATAL(t,NULL);

      free(pr);
    }

    freeTree(ptree);

}


/**
 * \test
 *
 * Testa la lettura/scrittura di un albero da file
 */
void loadstore_test (void)
{
    run_testleggiscrivi(FILE_TEST1,FILE_TEST2);
}
/**
 * \test
 *
 * Testa le funzioni di ptree.h
 */
void ptree_test (void)
{
  run_testAddCheckRemove();
}

/** 
 * Definizione della suite. 
 */
void add_suite_ptree(void)
{
    CU_pSuite ptree_suite = NULL;

    ptree_suite = CU_add_suite("Ptree Test", success_init, success_clean);
    CU_add_test(ptree_suite, "Test albero", ptree_test);
    CU_add_test(ptree_suite, "Test lettura scrittura albero da file", loadstore_test);

}

